
import sys
import time
import random
import math
import copy

import pygame


MIN_FRAME_TIME = 0.01 #seconds

GRAVITY = -50.0 #pixels/second^2
WING_FLAP_SPEED = 10 #pixels/second/feather
feather_count = 9
BIRDS_SPEED = 35.0 #pixels/second
TIME_BETWEEN_BIRD_HITS = 3.2 #seconds
FEATHER_CYCLE_SPEED = math.pi #radians/second
FEATHER_CYCLE_SIZE_X = 30 #pixels
FEATHER_CYCLE_SIZE_Y = 10 #pixels
ICARUS_HIT_IMAGE_DURATION = 0.8 #seconds
GROUND_Y_POSITION = 0
ICARUS_GROUND_SPEED = 160

icarus_position_x = 0 # position of center of body
icarus_position_y = 0
speed_x = 0
speed_y = 0

position_ox = -60
game_over = False
zone_len = 6
degree = 30



class Feather:
	image = pygame.image.load("feather.png")
	def __init__(self, position_x, position_y):
		self.start_time = time.time()
		self.real_position_x = position_x
		self.real_position_y = position_y
		self.calc_position()
	def calc_position(self):
		feather_time = time.time()-self.start_time
		self.position_x = self.real_position_x + math.cos(FEATHER_CYCLE_SPEED * feather_time) * FEATHER_CYCLE_SIZE_X
		self.position_y = self.real_position_y + math.sin(FEATHER_CYCLE_SPEED * feather_time) * FEATHER_CYCLE_SIZE_Y
	def move(self, time_passed):
		self.real_position_y -= 55 * time_passed
		self.calc_position()
	def draw(self, screen, position_view_x, position_view_y):
		screen.blit(Feather.image,
			[self.position_x+position_view_x-18,
			 -self.position_y+position_view_y-30])

class Bird:
	image1 = pygame.image.load("bird1.PNG")
	image2 = pygame.image.load("bird2.PNG")
	def __init__(self, position_x, position_y):
		self.position_x = position_x
		self.position_y = position_y
		self.is_hit = False
		self.last_time_the_bird_hit_icarus = time.time()
	def move(self, time_passed):
		self.position_x -= BIRDS_SPEED * time_passed
	def draw(self, screen, position_view_x, position_view_y):
		if int(time.time()*5) %2 == 0:
			image_to_draw = Bird.image1
		else:
			image_to_draw = Bird.image2
		screen.blit(image_to_draw,
					[self.position_x+position_view_x-49,
					 -self.position_y+position_view_y-73])

class Platform:
	image = pygame.image.load("island.png")
	end_flag_image = pygame.image.load("end.png")
	def __init__(self, position_x, has_flag):
		self.position_x = position_x
		self.has_flag = has_flag
	def draw(self, screen, position_view_x, position_view_y):
		screen.blit(Platform.image,[self.position_x+position_view_x,40+ position_view_y])
		if self.has_flag:
			screen.blit(Platform.end_flag_image,[self.position_x+position_view_x+160,-100+ position_view_y])

size = screen_width, screen_height = 800, 600
screen = pygame.display.set_mode ((size))
hero1 = pygame.image.load("icarus1.png")
hero2 = pygame.image.load("icarus2.png")
hero_hit = pygame.image.load("icarus_hit.png")
background = pygame.image.load("blue_ocean.png")
ox = pygame.image.load("ox.png")
sky = pygame.image.load("upper_background.png")
over = pygame.image.load("game_over.png")

feathers = []
birds = []
platforms = []
platforms.append(Platform(-160,False))
platforms.append(Platform((zone_len-0.5)*800,True))

time_between_birds = time.time() + 3
for i in range(5):
	birds.append(Bird(random.randint(1000,4000),random.randint(100,500)))

is_pressed_right = False
is_pressed_left = False
is_pressed_up = False

begining_time = time.time()
last_time_icarus_was_hit = 0


def is_on_platform():
	if icarus_position_y > GROUND_Y_POSITION:
		return False
	for platform in platforms:
		if icarus_position_x >= platform.position_x+25 and icarus_position_x <= 340 +platform.position_x:
			return True
	return False

last_time = time.time()
while True:
	
	curr_time = time.time()
	time_passed = curr_time - last_time
	last_time = curr_time
	
	if time_passed < MIN_FRAME_TIME:
		time.sleep(MIN_FRAME_TIME - time_passed)
	
	for event in pygame.event.get():
		if event.type == pygame.QUIT:
			sys.exit()
		if event.type == pygame.KEYDOWN and event.key == pygame.K_p :
			print icarus_position_y
		
		if not game_over:
			if event.type == pygame.KEYDOWN and event.key == pygame.K_RIGHT :
				is_pressed_right = True
			if event.type == pygame.KEYDOWN and event.key == pygame.K_LEFT :
				is_pressed_left = True	
			if event.type == pygame.KEYUP and event.key == pygame.K_RIGHT:
				is_pressed_right = False
			if event.type == pygame.KEYUP and event.key == pygame.K_LEFT:
				is_pressed_left = False
			
			actual_flap_speed = feather_count*WING_FLAP_SPEED
			if event.type == pygame.KEYDOWN and event.key == pygame.K_UP:
				speed_y += actual_flap_speed
				is_pressed_up = True
			if event.type == pygame.KEYUP and event.key == pygame.K_UP:	
				is_pressed_up = False
				if is_pressed_right :
					speed_x += actual_flap_speed
				if is_pressed_left :
					speed_x -= actual_flap_speed
			if event.type == pygame.KEYDOWN and event.key == pygame.K_DOWN:
				speed_y -= actual_flap_speed
				if is_pressed_right :
					speed_x += actual_flap_speed
				if is_pressed_left :
					speed_x -= actual_flap_speed
	
	if game_over:
		icarus_position_x += speed_x * time_passed
	else:
		if not is_on_platform():
			speed_x *= math.exp(-1*time_passed)
			icarus_position_x += speed_x * time_passed
		else:
			speed_x = 0
			if is_pressed_right:
				icarus_position_x += ICARUS_GROUND_SPEED * time_passed
			if is_pressed_left:
				icarus_position_x -= ICARUS_GROUND_SPEED * time_passed
	
	# standing on the flatform
	if (not game_over) and is_on_platform() and speed_y<=0:
		icarus_position_y = GROUND_Y_POSITION
		speed_y = 0
	#or falling
	else:
		speed_y += GRAVITY * time_passed
		icarus_position_y += speed_y * time_passed

	if not game_over:	
		#falling to the water:
		if icarus_position_y < -50:
			game_over = True
			speed_y = 150
			speed_x = -80
			GRAVITY = -260
		
	if time.time() > time_between_birds :
		birds.append(Bird(random.randint(500,4000),random.randint(100,400)))
		time_between_birds += 5
	
	#checking hitting a bird
	for bird in birds:
		#relative position from icarus:
		rel_x = icarus_position_x - bird.position_x
		rel_y = icarus_position_y - bird.position_y
		#from above:
		#50=size of rect around the centers of icarus and the bird
		#-90=addition of length from center to feet of icarus
		if abs(rel_x) < 50:
			if abs(rel_y-90) < 50:
				if not bird.is_hit: #makes sure only one feather is created
					bird.is_hit = True
					feathers.append(Feather(bird.position_x, bird.position_y))
		#from below:
		#this time we take +20 as length from center to head
		if abs(rel_x) < 50:
			if abs(rel_y+20) < 50:
				if time.time() - bird.last_time_the_bird_hit_icarus > TIME_BETWEEN_BIRD_HITS:
					bird.last_time_the_bird_hit_icarus = time.time()
					last_time_icarus_was_hit = time.time()
					print "lost a feather"
					if feather_count > 0:
						feather_count -= 1
						#lose the feather from the bottom, so it will be hard to reach
						feathers.append(Feather(icarus_position_x, icarus_position_y-120))
					
	
	#checking if catching a feather
	for feather in copy.copy(feathers):
		#relative position from icarus:
		rel_x = icarus_position_x - feather.position_x
		rel_y = icarus_position_y - feather.position_y
		if abs(rel_x) < 50:
			if abs(rel_y+20) < 50:
				print "caught a feather"
				feathers.remove(feather)
				feather_count += 1
	
	
	for feather in feathers:
		feather.move(time_passed)
	
	#move birds:
	for bird in birds:
		bird.move(time_passed)
	
	#move the minotaur:
	if begining_time+3 < curr_time :
		position_ox += 20 * time_passed
	
	# ################## #
	#      DRAWING:      #
	# ################## #
	
	#how much to move everything in the world:
	position_view_x = screen_width/2 - icarus_position_x
	position_view_y = screen_height/2 + icarus_position_y
	
	if position_view_y <0 and False:
		position_view_y = 0
	
	world_width = 6*800
	position_view_x = min(position_view_x, 0)
	position_view_x = max(position_view_x, screen_width-world_width)
	position_view_y = max(position_view_y, screen_height/2)
	
	screen.fill ((0,255,255))
	
	#draw the background: sea
	screen.blit(background,[position_view_x%800-800,position_view_y-300])
	screen.blit(background,[position_view_x%800    ,position_view_y-300])
		
	#sun, up at the sky
	screen.blit(sky,[60,position_view_y-1000])
	
	#islands
	for platform in platforms:
		platform.draw(screen, position_view_x, position_view_y)	
	
	if game_over:
		screen.blit(over,[300,100])
	
	# draw the feathers
	for feather in feathers:
		feather.draw(screen, position_view_x, position_view_y)
	
	#draw birds with wing flapping:
	for bird in birds:
		bird.draw(screen, position_view_x, position_view_y)
	
	#fethers count display:
	
	#pygame.font.init()
	#font = pygame.font.Font(None, 36)
	#text = font.render("Your current speed rate: %d" % (feather_count),True, (255,255,255),(0,0,0))
	#textRect = text.get_rect()
	#screen.blit(text, textRect)
	for i in xrange(feather_count):
		screen.blit(Feather.image, (700-5*i, 20))
	
	#draw hero with wing flapping
	if time.time()-last_time_icarus_was_hit < ICARUS_HIT_IMAGE_DURATION:
		hero_image = hero_hit
	elif game_over:
		hero_image = hero_hit
	elif begining_time+3 < curr_time :
		if int(time.time()*1.5) %2 == 0:
			hero_image = hero1
		else:
			hero_image = hero2
	else:
		hero_image = hero1
	screen.blit(hero_image,[icarus_position_x+position_view_x-87,-icarus_position_y+position_view_y-71])

	pygame.display.flip()